// $Id:$   -*-c++-*-
// by Chris Crawford <chris2@lns.mit.edu> 2005/03/16
// 
// wc_cuts
//   plots for elastic wc cuts
// 
// root -l wc_cuts.C r/ 53-88 +plot 
// root -l wc_cuts.C +show
// root -l wc_cuts.C r/ 55-88 +offsets
// root -l wc_cuts.C r/ 55-88 +q2res
//

#include "/home/blast/chris2/blast/o/init.C"

bool plot=false;
bool show=false;
bool offsets=false;
bool q2res=false;

void wc_cuts()
{
  init("hpt");
  gROOT->Macro("/home/blast/chris2/blast/o/cuts_h3.C+");
  if (plot) wc_plot();
  if (show) wc_show();
  if (offsets) wc_offsets();
  if (q2res) wc_q2res();
}


void wc_plot()
{
  TPad* p=InitCanvas("wc","Elastic WC Cuts",6,5);
  char* cl="qwl==-1&&qwr==1";
  char* cr="qwl==1&&qwr==-1";
  
  // p:th electron
  p->cd(1);  chain[0]->Draw(pwe+"-"+pke+":"+twe+">>h_pe_l(53,23,76,100,-.2,.2)",cl,"col", nevents);
  p->cd(4);  chain[0]->Draw(pwe+"-"+pke+":"+twe+">>h_pe_r(53,23,76,100,-.2,.2)",cr,"col", nevents);

  // p:th proton
  p->cd(7);  chain[0]->Draw(pwp+"-"+pkp+":"+twp+">>h_pp_l(53,23,76,100,-.2,.2)",cl,"col", nevents);
  p->cd(10); chain[0]->Draw(pwp+"-"+pkp+":"+twp+">>h_pp_r(53,23,76,100,-.2,.2)",cr,"col", nevents);

  // th_p:th_e
  p->cd(13); chain[0]->Draw(twp+"-"+tkp+":"+twe+">>h_tt_l(53,23,76,100,-3,3)",cl,"col", nevents);
  p->cd(16); chain[0]->Draw(twp+"-"+tkp+":"+twe+">>h_tt_r(53,23,76,100,-3,3)",cr,"col", nevents);

  // f_p:f_e
  p->cd(19); chain[0]->Draw(fwp+"-"+fwe+":"+fwe+">>h_ff_l(28,-14,14,100,-8,8)",cl,"col", nevents);
  p->cd(22); chain[0]->Draw(fwp+"-"+fwe+":"+fwe+">>h_ff_r(28,-14,14,100,-8,8)",cr,"col", nevents);

  // z_p:z_e
  p->cd(25); chain[0]->Draw(zwp+"-"+zwe+":"+zwe+">>h_zz_l(40,-20,20,100,-10,10)",cl,"col", nevents);
  p->cd(28); chain[0]->Draw(zwp+"-"+zwe+":"+zwe+">>h_zz_r(40,-20,20,100,-10,10)",cr,"col", nevents);
}

TPad* p;
char* hl[2][2] = {{"pe_l","pe_r"},{"pp_l","pp_r"}};
char* hr[2][2] = {{"(60,.50,.80, 100,-.2,.2)", "(60,.52,.82, 100,-.2,.2)"},
		  {"(60,.30,.90, 100,-.2,.2)", "(55,.30,.85, 100,-.2,.2)"}};
char* ht[2][2] = {{"Left Electron ", "Right Electron"},
		  {"Left Proton   ", "Right Proton  "}};

wc_offsets()
{
  gStyle->SetOptStat(0);
  p=InitCanvas("wo","WC Elastic Momentum Offsets = p_{wc} - p_{calc}(#theta) : p_{wc}",2,2);

  TCut    hc[4] = { Hwd_L(), Hwd_R(), Hwd_R(), Hwd_L() };
  TString hd[2] = { pwe+"-"+pke+":"+pwe, pwp+"-"+pkp+":"+pwp };

  for (int ep=0;ep<2;ep++) for (int lr=0;lr<2;lr++) {
    p->cd(2*ep+lr+1);
    chain[0]->Draw(hd[ep]+">>"+hl[ep][lr]+hr[ep][lr],
		   hc[2*ep+lr], "col", nevents);
    TH2* hh=gDirectory->Get(hl[ep][lr]);
    hh->SetTitle(ht[ep][lr]);
    hh->FitSlicesY();
    NormalizeY(hh);
    wc_ofit(ep,lr);
    TPad::Pad()->Update();
  }
  p->cd();
}

wc_ofit(int ep, int lr, char* fn=0, char* fopt="e")
{
  char* fna[2][2]={{"pol4","pol4"},{"pol5","pol5"}};
  if (!fn) fn=fna[ep][lr];
  p->cd(2*ep+lr+1);
  TH1* hm=gDirectory->Get(hl[ep][lr]+TString("_1"));
  hm->SetLineColor(kRed);
  hm->Fit(fn, fopt, "same");
  TF1* hf=hm->GetFunction(fn);
  hf->SetLineColor(kBlue);
  hf->SetLineWidth(1);
}

wc_osave(char* filename="wc_offsets_p.root")
{
  TFile f(filename,"create");
  p->GetCanvas()->Write();
  for (int ep=0;ep<2;ep++) for (int lr=0;lr<2;lr++)
    gROOT->Get(hl[ep][lr])->Write();
  f.Close();
}

wc_q2res()
{
  gStyle->SetOptStat(0);
  p=InitCanvas("wq","WC Q^{2} Systematics   Q^{2}(#theta_{e}) - Q^{2}(#theta_{p}) : Q^{2}(#theta_{e},#theta_{p})",2,1);

  TCut hc[2] = { Hwd_L(), Hwd_R() };
  for (int lr=0;lr<2;lr++) {
    p->cd(lr+1);
    TString drw=q2te+"-"+q2tp+":"+q2k+">>"+hm[lr]+"(50,.11,.66)";
    chain[0]->Draw(drw, hc[lr], "col", nevents);
    TH2* hh=gDirectory->Get(hm[lr]);
    hh->SetTitle(ht[0][lr]);
    hh->FitSlicesY();
    NormalizeY(hh);
    TPad::Pad()->Update();
  }
  p->cd();
}

 
Double_t draw_fn(Double_t *x, Double_t *p)
{
  // p[0]={0=pe,1=pp,2=tt,3=ff,4=zz}, p[1]=electron sector, p[2]={n_sigma}
  if (p[0]==0) return pde_(*x,*x,p[1]) + p[2]*pse_(*x,*x,p[1]);
  if (p[0]==1) return pdp_(*x,*x,p[1]) + p[2]*psp_(*x,*x,p[1]);
  if (p[0]==2) return tdp_(*x,*x,p[1]) + p[2]*tsp_(*x,*x,p[1]);
  if (p[0]==3) return fdp_(*x,*x,p[1]) + p[2]*fsp_(*x,*x,p[1]);
  if (p[0]==4) return zdp_(*x,*x,p[1]) + p[2]*zsp_(*x,*x,p[1]);
  return 0;
}

void wc_show()
{
  // draw 5 canvases each with histogram and -2,1,0,1,2 sigma lines
  
  TFile* f=new TFile("wc_cuts_j3.root");
  TCanvas* c[5];
  char* vn[5]={"pe","pp","tt","ff","zz"};
  char* sn[2]={"l","r"};

  for (int j=0;j<5;j++) {
    c[j]=new TCanvas(vn[j],vn[j]);
    c[j]->Divide(2,1);
    for (int k=0;k<2;k++) {
      c[j]->cd(k+1);

      TH2* h=(TH2*)f->Get(Form("h_%s_%s",vn[j],sn[k]));
      h->Draw("col");
      for(int i=-2;i<=2;i++) {
	TF1* fn = new TF1(Form("f_%s_%s_%d",vn[j],sn[k],i), draw_fn, -20,100,3);
	fn->SetParameters(j,k,i);
	fn->SetLineColor(kRed);
	fn->SetLineWidth(1);
	fn->Draw("same");
      }
    }
  }
}

// root -l wc_cuts.C r/ 53-57 -n 10000
void compare()
{
  init("hpt");
  gROOT->Macro("/home/blast/chris2/blast/o/cuts_h3.C+");
  TPad* p=InitCanvas("wc","Elastic WC Cuts",6,5);
  char* cl="qwl==-1&&qwr==1";
  char* cr="qwl==1&&qwr==-1";
  
  // p:th electron
  p->cd(1);  chain[0]->Draw(pwe+":"+twe,cl,"col", nevents);
  p->cd(4);  chain[0]->Draw(pwe+":"+twe,cr,"col", nevents);

  // p:th proton
  p->cd(7);  chain[0]->Draw(pwp+":"+twp,cl,"col", nevents);
  p->cd(10); chain[0]->Draw(pwp+":"+twp,cr,"col", nevents);

  // th_p:th_e
  p->cd(13); chain[0]->Draw(twp+":"+twe,cl,"col", nevents);
  p->cd(16); chain[0]->Draw(twp+":"+twe,cr,"col", nevents);

  // f_p:f_e
  p->cd(19); chain[0]->Draw(fwp+":"+fwe,cl,"col", nevents);
  p->cd(22); chain[0]->Draw(fwp+":"+fwe,cr,"col", nevents);

  // z_p:z_e
  p->cd(25); chain[0]->Draw(zwp+":"+zwe,cl,"col", nevents);
  p->cd(28); chain[0]->Draw(zwp+":"+zwe,cr,"col", nevents);

  chain[0]->SetMarkerColor(kRed);

  // p:th electron
  p->cd(1);  chain[0]->Draw(pke+":"+twe,cl,"histsame", nevents);
  p->cd(4);  chain[0]->Draw(pke+":"+twe,cr,"histsame", nevents);
			       
  // p:th proton	       
  p->cd(7);  chain[0]->Draw(pkp+":"+twp,cl,"histsame", nevents);
  p->cd(10); chain[0]->Draw(pkp+":"+twp,cr,"histsame", nevents);
			       
  // th_p:th_e		       
  p->cd(13); chain[0]->Draw(tkp+":"+twe,cl,"histsame", nevents);
  p->cd(16); chain[0]->Draw(tkp+":"+twe,cr,"histsame", nevents);
			       
  // f_p:f_e		       
  p->cd(19); chain[0]->Draw(fwe+":"+fwe,cl,"histsame", nevents);
  p->cd(22); chain[0]->Draw(fwe+":"+fwe,cr,"histsame", nevents);
			       
  // z_p:z_e		       
  p->cd(25); chain[0]->Draw(zwe+":"+zwe,cl,"histsame", nevents);
  p->cd(28); chain[0]->Draw(zwe+":"+zwe,cr,"histsame", nevents);
}

TObject* Find(TString name, TObject* base=gDirectory)
{
  // recursively find the object
  if (base->InheritsFrom("TKey"))
    return Find( name, ((TKey*)base)->ReadObj() );
  if (base->GetName() == name)
    return base;

  // search directory, pad, etc
  TList* seq=0;
  if (base->InheritsFrom("TList")) seq=base;
  if (base->InheritsFrom("TDirectory"))
    seq=((TDirectory*)base)->GetListOfKeys();
  if (base->InheritsFrom("TPad"))
    seq=((TPad*)base)->GetListOfPrimitives();
  if (seq) {
    TIter next(seq); TObject *obj, *ret;
    while ((obj=next()))
      if ((ret=Find(name,obj)))
	return ret;
  }

  // none found
  return 0;
}
